/*-
 * Copyright (c) 2015
 *      Bill Paul <wpaul@windriver.com>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Bill Paul.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdint.h>
#include <avr/io.h>

#include "dds.h"
#include "dds_in.h"
#include "dds_out.h"

/*
 * Kenwood F71022 DDS chip emulation firmware
 * Written by Bill Paul N1GPT
 */

volatile static const char rcsid[] =
	"$Id: dds.c,v 1.21 2016/01/31 20:02:13 wpaul Exp $";

/*
 * On the Arduino Mini v5 prototype board, the AVR I/O ports correspond
 * to the following pins:
 *
 * PORTB.0 IO8			(1/2 clock out)
 * PORTB.1 IO9			(input clock)
 * PORTB.2 IO10			(input data)
 * PORTB.3 IO11			(input select)
 *
 * PORTC.0 A0			(output clock0)
 * PORTC.1 A1			(output data0)
 * PORTC.2 A2			(output select0)
 *
 * PORTD.0 			(output clock1)
 * PORTD.1 			(output data1)
 * PORTD.2 			(output select1)
 */

int main(void)
{
	/*
	 * Set up the clock prescaler to divide the supplied
	 * input clock by two.
	 */

	CLKPR = 1 << CLKPCE;
	CLKPR = 1 << CLKPS0;

	/* Disable pullup resistors. */

	MCUCR |= 1 << PUD;

	/*
         * PORTB pins 1, 2, 3 and 4 are set as inputs. (Pin
	 * 0 is used as CKOUT signal.)
	 * Set direction bits to 0.
	 * Note: we also have to keep bits 6 and 7 set as inputs.
	 * When testing boards that are still set to use the AVR's
	 * internal clock, leaving pin 6 set as an output will cause
	 * it to drain a lot of current from the external clock source.
	 * This will prevent the DDS chips from working.
	 */

	DDRB = 0x21;
	PORTB = 0x00;

	/* Port C is set to all outputs */

	PORTC = 0xFF;
	DDRC = 0xFF;

	/* Port D is set to all outputs */

	PORTD = 0xFF;
	DDRD = 0xFF;

	/*
	 * We have to initialize the AD9832 chips at startup. One hopes
	 * we have enough time to do this before the first command from
	 * the computer arrives.
	 */

	dds_out_init();

	/* Initialize DDS input module */

	dds_in_init();

	while (1) {
		dds_in_scan();
		dds_out_send();
	}

	/* NOTREACHED */
	return (0);
}
